﻿using System;
using System.Collections.Generic;
using JetBrains.Application;
using JetBrains.ReSharper.Psi;
using JetBrains.ReSharper.Psi.CSharp;
using JetBrains.ReSharper.Psi.CSharp.Tree;
using JetBrains.ReSharper.Psi.ExtensionsAPI.Tree;
using JetBrains.ReSharper.Psi.Modules;
using JetBrains.ReSharper.Psi.Tree;

namespace Recogmission.ResharperPlugin.CodeCleanup
{
    /// <summary>
    /// Base code cleanups.
    /// </summary>
    internal class BaseCodeCleanups
    {
        private static readonly Dictionary<String, String> aliasesDictionary = new Dictionary<string, string>
        {
            {"bool", "Boolean"},
            {"byte", "Byte"},
            {"char", "Char"},
            {"double", "Double"},
            {"decimal", "Decimal"},
            {"float", "Single"},
            {"int", "Int32"},
            {"sbyte", "SByte"},
            {"short", "Int16"},
            {"object", "Object"},
            {"string", "String"},
            {"ushort", "UInt16"},
            {"uint", "UInt32"},
            {"long", "Int64"},
            {"ulong", "UInt64"},
        };

        /// <summary>
        /// Executes the cleanup rules.
        /// </summary>
        /// <param name="file">
        ///     The file to process.
        /// </param>
        /// <param name="shellLocks"></param>
        /// <param name="options">
        /// The options.
        /// </param>
        public void Execute( ICSharpFile file, IShellLocks shellLocks )
        {
            try
            {

                file.GetPsiServices().Transactions.Execute(
                    "Code Cleanup",
                    () => 
                         {
                             using (shellLocks.UsingWriteLock())
                             {
                                 SwapToBuiltInTypeAlias(file.FirstChild);
                             }
                         }
                    );
            }
            catch (Exception ex)
            {
                Console.WriteLine("Ex {0}", ex.Message);
            }
        }

        /// <summary>
        /// Swap to CLR type name.
        /// </summary>
        /// <param name="node">
        ///     The node to process.
        /// </param>
        /// <param name="shellLocks"></param>
        public static void SwapToBuiltInTypeAlias( ITreeNode node )
        {
            for( ITreeNode currentNode = node; currentNode != null; currentNode = currentNode.NextSibling )
            {
                var predefinedTypeReference = currentNode as IPredefinedTypeReference;

                if( predefinedTypeReference != null )
                {
                    String typeName = predefinedTypeReference.Reference.GetName();
                    IPsiModule project = currentNode.GetPsiModule();
                    String clrTypeName;

                    if( aliasesDictionary.TryGetValue( typeName, out clrTypeName ) )
                    {
                        Boolean isEnum = false;
                        if( currentNode.Parent != null )
                        {
                            if( currentNode.Parent.Parent is IEnumBase )
                            {
                                isEnum = true;
                            }
                        }

                        if( !isEnum )
                        {
                            var newTypeUsageNode =
                                CSharpElementFactory.GetInstance( project ).CreateReferenceName( clrTypeName );

                            try
                            {
                                ModificationUtil.ReplaceChild( node, (ITreeNode)newTypeUsageNode );
                                
                            }
                            catch( Exception ex )
                            {
                                Console.WriteLine("Ex {0}", ex.Message);
                            }
                        }
                    }
                }

                if( currentNode.FirstChild != null )
                {
                    SwapToBuiltInTypeAlias( currentNode.FirstChild );
                }
            }
        }
    }
}