﻿using System;
using System.Collections.Generic;
using System.Linq;
using JetBrains.Application;
using JetBrains.Application.Progress;
using JetBrains.DocumentModel;
using JetBrains.ReSharper.Feature.Services.CodeCleanup;
using JetBrains.ReSharper.Feature.Services.CSharp.CodeCleanup;
using JetBrains.ReSharper.Psi;
using JetBrains.ReSharper.Psi.CSharp;
using JetBrains.ReSharper.Psi.CSharp.Tree;
using JetBrains.ReSharper.Psi.Files;

namespace Recogmission.ResharperPlugin.CodeCleanup
{
    /// <summary>
    /// ReSharper CodeCleanUp.
    /// </summary>
    [CodeCleanupModule(
        ModulesBefore =
            new[]
            {
                typeof( UpdateFileHeader ), typeof( ArrangeThisQualifier ), typeof( ReplaceByVar ),
                typeof( ReformatCode )
            }
        )]
    public class ReSharperCodeCleanupModule : ICodeCleanupModule
    {
        #region Constants and Fields

        private readonly IShellLocks _shellLocks;

        /// <summary>
        /// Readability descriptor.
        /// </summary>
        private static readonly BaseCodeCleanupsDescriptor baseCodeCleanupsDescriptor = new BaseCodeCleanupsDescriptor();

        #endregion

        #region Constructor
        /// <summary>
        /// </summary>
        /// <param name="shellLocks"></param>
        public ReSharperCodeCleanupModule(IShellLocks shellLocks)
        {
          _shellLocks = shellLocks;
        }

        #endregion


        #region Properties

        /// <summary>
        /// Gets the collection of option descriptors.
        /// </summary>
        /// <value>
        /// The descriptors.
        /// </value>
        public ICollection<CodeCleanupOptionDescriptor> Descriptors
        {
            get
            {
                return new CodeCleanupOptionDescriptor[] { baseCodeCleanupsDescriptor };
            }
        }

        /// <summary>
        /// Gets a value indicating whether the module is available on selection, or on the whole file.
        /// </summary>
        /// <value>
        /// The is available on selection.
        /// </value>
        public bool IsAvailableOnSelection
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Gets the language this module can operate.
        /// </summary>
        /// <value>
        /// The language type.
        /// </value>
        public PsiLanguageType LanguageType
        {
            get
            {
                return CSharpLanguage.Instance;
            }
        }

        #endregion

        #region Implemented Interfaces

        #region ICodeCleanupModule

        /// <summary>
        /// Process clean-up on file.
        /// </summary>
        /// <param name="rangeMarker"></param>
        /// <param name="profile">
        /// The code cleanup settings to use.
        /// </param>
        /// <param name="progressIndicator">
        /// The progress indicator.
        /// </param>
        /// <param name="sourceFile"></param>
        public void Process(
            IPsiSourceFile sourceFile, IRangeMarker rangeMarker, CodeCleanupProfile profile,
            IProgressIndicator progressIndicator )
        {
            var psiFiles = sourceFile.GetPsiServices().Files;

            ICSharpFile file = psiFiles.GetDominantPsiFile<CSharpLanguage>(sourceFile) as ICSharpFile; ;
            if( file == null )
            {
                return;
            }

            if( !profile.GetSetting( baseCodeCleanupsDescriptor ) )
            {
                return;
            }


            // Process the file for all the different Code Cleanups we have here
            // we do them in a very specific order. Do not change it.
            new BaseCodeCleanups().Execute( file, _shellLocks );
        }

        /// <summary>
        /// Get default setting for given profile type.
        /// </summary>
        /// <param name="profile">
        /// The code cleanup profile to use.
        /// </param>
        /// <param name="profileType">
        /// Determine if it is a full or reformat <see cref="CodeCleanup.DefaultProfileType"/>.
        /// </param>
        public void SetDefaultSetting(
            CodeCleanupProfile profile,
            JetBrains.ReSharper.Feature.Services.CodeCleanup.CodeCleanup.DefaultProfileType profileType )
        {
            switch (profileType)
            {
                case JetBrains.ReSharper.Feature.Services.CodeCleanup.CodeCleanup.DefaultProfileType.FULL:
                    profile.SetSetting(baseCodeCleanupsDescriptor, true);
                    break;

                case JetBrains.ReSharper.Feature.Services.CodeCleanup.CodeCleanup.DefaultProfileType.REFORMAT:
                    profile.SetSetting(baseCodeCleanupsDescriptor, true);
                    break;
                default:
                    throw new ArgumentOutOfRangeException("profileType");
            }
        }

        public bool IsAvailable( IPsiSourceFile sourceFile )
        {
            return sourceFile.GetPsiFiles<CSharpLanguage>().Any();
        }

        #endregion

        #endregion
    }
}